#
# Caller must have already done the following (at a minimum), to use the macros
# in this file.
#
#find_package(Orocos-RTT 1.6.0 REQUIRED)
#find_package(Orocos-OCL 1.6.0 REQUIRED)
#
# If the caller requires additional RTT and/or OCL components, they can be 
# added to the above e.g.
#
#find_package(Orocos-RTT 1.6.0 REQUIRED corba)
#find_package(Orocos-OCL 1.6.0 REQUIRED taskbrowser)
#
################################################################################


#
# - Create an Orocos component from one or more cpp files.
# Usage:
#  create_component(<component-name> [VERSION <version>] <files> )
#
# Example:
#  create_component(quicky VERSION 1.4 quicky.cpp)
#
# Creates the quicky library with version 1.4 (if supported by your OS) from the file quicky.cpp
#
# The VERSION argument is the version of your component, not the RTT version.
# This macro defaults to creating components with RTT/OCL 1.6.0 or later.
# If you want another version of OrocosRTT, call find_package(OrocosRTT <my.rtt.version> REQUIRED)
# before you call this macro.
#
# REQUIRES
#find_package(Orocos-RTT 1.6.0 REQUIRED)
#find_package(Orocos-OCL 1.6.0 REQUIRED)

macro(create_component COMPONENT_NAME ) # [VERSION x.y.z] files ...

  SET(_files)
  message( "Building shared library for ${COMPONENT_NAME}" )

  foreach( arg ${ARGN} )
    if ( ${arg} STREQUAL VERSION )
      SET(IN_VERSION 1)   
    else()
      if(IN_VERSION)
	#Process version argument and store result for use in set_target_properties
	set(_VERSION ${arg})
	string( REGEX MATCHALL "[0-9]+" _VERSIONS ${_VERSION} )
	list( GET _VERSIONS 0 _VERSION_MAJOR)
	list( GET _VERSIONS 1 _VERSION_MINOR)
	list( GET _VERSIONS 2 _VERSION_PATCH)
	list(LENGTH _VERSIONS vlength)
	if ( ${vlength} EQUAL 1 )
	  set( VERSION_SPEC VERSION ${_VERSION} SOVERSION ${_VERSION_MAJOR} )
	elseif(${vlength} EQUAL 2)
	  set( VERSION_SPEC VERSION ${_VERSION} SOVERSION ${_VERSION_MAJOR} )
	elseif(${vlength} EQUAL 3)
	  set( VERSION_SPEC VERSION ${_VERSION} SOVERSION ${_VERSION_MAJOR}.${_VERSION_MINOR} )
	endif()

	set(IN_VERSION 0)
      else()
	list(APPEND _files ${arg})
      endif()
    endif()
  endforeach()

  add_library( ${COMPONENT_NAME} SHARED ${_files} )
  include_directories(${OROCOS-RTT_INCLUDE_DIRS})
  set_target_properties( ${COMPONENT_NAME} PROPERTIES 
    DEFINE_SYMBOL OCL_DLL_EXPORT 
    COMPILE_DEFINITIONS "${OROCOS-RTT_DEFINITIONS}"
    ${VERSION_SPEC}
    )
  target_link_libraries( ${COMPONENT_NAME} ${OROCOS-RTT_LIBRARIES} ${OROCOS-OCL_LIBRARIES} )

endmacro()

#
# - Create an Orocos component from a given cpp file.
# Usage:
#  create_executable(<component-name> [VERSION <version>] <files> )
#
# Example:
#  create_executable(quicky quicky.cpp)
#
# Creates the quicky executable from the file quicky.cpp
#
# This macro defaults to creating components with RTT/OCL 1.6.0 or later.
# If you want another version of OrocosRTT, call find_package(OrocosRTT <my.rtt.version> REQUIRED)
# before you call this macro.
#
# REQUIRES
#find_package(Orocos-RTT 1.6.0 REQUIRED)
#find_package(Orocos-OCL 1.6.0 REQUIRED)

macro(create_executable EXECUTABLE_NAME ) # files ...

  message( "Building executable ${EXECUTABLE_NAME}" )

  add_executable( ${EXECUTABLE_NAME} ${ARGN} )
  include_directories(${OROCOS-RTT_INCLUDE_DIRS})
  set_target_properties( ${EXECUTABLE_NAME} PROPERTIES 
    COMPILE_DEFINITIONS "${OROCOS-RTT_DEFINITIONS}"
    )
  # forcibly link all RTT and OCL libraries that we found
  target_link_libraries( ${EXECUTABLE_NAME} ${OROCOS-RTT_LIBRARIES} ${OROCOS-OCL_LIBRARIES} )

endmacro()